package nl.v2b.lalaland.application;
import nl.v2b.lalaland.application.exceptions.CategoryNotFoundException;
import nl.v2b.lalaland.application.exceptions.ChallengeNotFoundException;
import nl.v2b.lalaland.application.exceptions.UnsupportedChallengeTypeException;
import nl.v2b.lalaland.data.SpringCategoryRepository;
import nl.v2b.lalaland.data.SpringChallengeRepository;
import nl.v2b.lalaland.domain.Category;
import nl.v2b.lalaland.domain.challenges.Challenge;
import nl.v2b.lalaland.domain.challenges.ChallengeType;
import nl.v2b.lalaland.domain.challenges.MediaChallenge;
import nl.v2b.lalaland.domain.challenges.TextChallenge;
import nl.v2b.lalaland.presentation.dtos.ChallengeDTO;
import nl.v2b.lalaland.storage.application.LalaFileService;
import nl.v2b.lalaland.storage.domain.LalaFile;
import org.springframework.stereotype.Service;

import jakarta.transaction.Transactional;
import java.util.List;


@Service
@Transactional
public class ChallengeService {
    private final SpringChallengeRepository repository;
    private final SpringCategoryRepository categoryRepository;
    private final LalaFileService lalaFileService;

    public ChallengeService(SpringChallengeRepository repository, SpringCategoryRepository categoryRepository, LalaFileService lalaFileService){
        this.repository = repository;
        this.lalaFileService = lalaFileService;
        this.categoryRepository = categoryRepository;
    }

    public List<Challenge> getAllChallenges() {
        return repository.findAll();
    }

    public Challenge createNewChallenge (ChallengeDTO dto){
        LalaFile lalaFile=null;
        Category category = categoryRepository.findById(dto.category).orElseThrow(ChallengeNotFoundException::new);
        if(dto.file!=null) {
            lalaFile=lalaFileService.get(dto.file);
        }
        if(dto.type==ChallengeType.Text){
            Challenge challenge = new TextChallenge(dto.point,
                    dto.description,
                    dto.title,
                    dto.answers,
                    lalaFile,
                    category
                    );
            return repository.save(challenge);
        }else if(dto.type==ChallengeType.Media){
            Challenge challenge = new MediaChallenge(dto.point, dto.description,
                    dto.title,lalaFile,category);
            return repository.save(challenge);
        } else throw new UnsupportedChallengeTypeException();
    }

    public Challenge updateChallenge(ChallengeDTO dto, int id){
        if (!getAllChallenges().contains(getChallenge(id)))throw new ChallengeNotFoundException();

        LalaFile lalaFile = null;
        if(dto.file!=null) {
            lalaFile=lalaFileService.get(dto.file);
        }
        Category category = categoryRepository.findById(dto.category).orElseThrow(ChallengeNotFoundException::new);
        if(dto.type==ChallengeType.Text){
            Challenge challenge = new TextChallenge(dto.point,
                    dto.description,
                    dto.title,
                    dto.answers,
                    lalaFile,
                    category
            );
            challenge.setId(id);
            return repository.save(challenge);
        }else if(dto.type==ChallengeType.Media){
            Challenge challenge = new MediaChallenge(dto.point, dto.description,
                    dto.title,lalaFile, category);
            challenge.setId(id);
            return repository.save(challenge);
        } else throw new UnsupportedChallengeTypeException();
    }

    public Challenge getChallenge(int id){
        return repository.findById(id).orElseThrow(ChallengeNotFoundException::new);
    }

    public void deleteChallenge (int id){
        if (getAllChallenges().contains(getChallenge(id)))
            repository.deleteById(id);
    }

    private boolean categoryExistsCheck(int id) {
        return this.categoryRepository.findById(id).isPresent();
    }

    public void addChallengeToCategory(int id, int idChallenge){
        this.categoryRepository.findById(id).orElseThrow(CategoryNotFoundException::new)
                .getChallenges().add(repository.findById(idChallenge).orElseThrow(ChallengeNotFoundException::new));
    }

    public void removeChallengeFromCategory(int id, int idChallenge) {
        this.categoryRepository.findById(id).orElseThrow(CategoryNotFoundException::new)
                .getChallenges().remove(repository.findById(idChallenge).orElseThrow(ChallengeNotFoundException::new));
    }
    public Challenge archiveChallenge(int id,boolean archive){
        Challenge challenge = getChallenge(id);
        challenge.setArchive(archive);
        return repository.save(challenge);
    }
}
