package nl.v2b.lalaland.application;

import nl.v2b.lalaland.application.exceptions.ChallengeAlreadyPassedException;
import nl.v2b.lalaland.application.exceptions.ChallengeNotFoundException;
import nl.v2b.lalaland.application.exceptions.SubmissionNotFoundException;
import nl.v2b.lalaland.data.SpringChallengeRepository;
import nl.v2b.lalaland.data.SpringImageSubmissionRepository;
import nl.v2b.lalaland.domain.Player;
import nl.v2b.lalaland.domain.State;
import nl.v2b.lalaland.domain.User;
import nl.v2b.lalaland.domain.challenges.MediaChallenge;
import nl.v2b.lalaland.domain.submission.ImageSubmission;
import nl.v2b.lalaland.domain.submission.Submission;
import nl.v2b.lalaland.storage.application.LalaFileService;
import nl.v2b.lalaland.storage.domain.LalaFile;
import org.springframework.stereotype.Service;

import jakarta.transaction.Transactional;
import java.util.List;

@Service("imageSubmissionService")
@Transactional
public class ImageSubmissionService {
	private final SpringImageSubmissionRepository imageRepository;
	private final SpringChallengeRepository challengeRepository;
	private final LalaFileService lalaFileService;

	public ImageSubmissionService(SpringImageSubmissionRepository imageRepository, SpringChallengeRepository challengeRepository, LalaFileService lalaFileService) {
		this.imageRepository = imageRepository;
		this.challengeRepository = challengeRepository;
		this.lalaFileService = lalaFileService;
	}

	public List<ImageSubmission> getAllImageSubmissionsByChallengeId(int id) {
		return imageRepository.findAllByChallenge_Id(id);
	}

	public List<ImageSubmission> getAllPendingSubmissionsByChallengeId(int id) {
		List<ImageSubmission> submissions = imageRepository.findAllByChallenge_Id(id);
		submissions.removeIf(submission -> submission.getState() != State.PENDING);
		return submissions;
	}

	public List<ImageSubmission> getAllImageSubmissionsByUserAndChallenge(int challengeId, User user) {
		return imageRepository.findAllByChallenge_IdAndPlayer_Id(challengeId, user.getId());
	}

	public List<ImageSubmission> getAllImageSubmissionsByUser(User user) {
		return imageRepository.findAllByPlayer_Id(user.getId());
	}

	public ImageSubmission submitImage(int challengeId, User user, String attempt) {
		if (!imageRepository.findAllByChallenge_IdAndPlayer_Id(challengeId, user.getId()).isEmpty()) {
			List<ImageSubmission> submissions = imageRepository.findAllByChallenge_IdAndPlayer_Id(challengeId, user.getId());
			for (ImageSubmission submission : submissions) {
				if (submission.getState() == State.PASSED)
					throw new ChallengeAlreadyPassedException("You've already passed this challenge");
				//todo: afvangen dat als challenge pending is dat er dan niet een nieuwe submissie gedaan wordt
			}
		}
		LalaFile lalaFile = null;
		if (attempt != null) lalaFile = lalaFileService.get(attempt);
		MediaChallenge challenge = (MediaChallenge) challengeRepository.findById(challengeId).orElseThrow(ChallengeNotFoundException::new);
		ImageSubmission submission = new ImageSubmission(lalaFile, challenge, (Player) user);
		return imageRepository.save(submission);
	}

	public ImageSubmission judgeSubmission(int submissionId, boolean accepted) {
		ImageSubmission imageSubmission = imageRepository.findById(submissionId).orElseThrow(SubmissionNotFoundException::new);
		if (accepted) {
			imageSubmission.setState(State.PASSED);
		} else imageSubmission.setState(State.FAILED);
		return imageRepository.save(imageSubmission);
	}
	public List<ImageSubmission> getAllImageSubmissions (){
	    return imageRepository.findAll();
    }
}

