package nl.v2b.lalaland.application;

import nl.v2b.lalaland.application.exceptions.UserBlockedException;
import nl.v2b.lalaland.application.exceptions.UserNotFoundException;
import nl.v2b.lalaland.data.SpringUserRepository;
import nl.v2b.lalaland.domain.Admin;
import nl.v2b.lalaland.domain.Player;
import nl.v2b.lalaland.domain.User;
import nl.v2b.lalaland.presentation.dtos.PlayerDTO;
import nl.v2b.lalaland.presentation.dtos.UserDTO;
import nl.v2b.lalaland.presentation.dtos.UserRequestDTO;
import org.hibernate.*;
import org.hibernate.query.Query;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.stereotype.Service;

import jakarta.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Service("userService")
@Transactional
public class UserService {
	private final SpringUserRepository repository;
	@Value("${lalaland.initial-admin}")
	private String initalAdminEmail;

	public UserService(SpringUserRepository repository) {
		this.repository = repository;
	}

	public Optional<User> getUser(String sub) {
		return repository.findByAuth0(sub);
	}

	/**
	 * This function is in use by: {@link nl.v2b.lalaland.security.CurrentUser} .
	 */
	public User getUser(OidcUser oidcUser) {
		User user = this.getUserOptional(oidcUser).orElseThrow(UserNotFoundException::new);
		if (user.isBlocked()) throw new UserBlockedException();
		return user;
	}

	public Optional<User> getUserOptional(OidcUser oidcUser) {
		return repository.findByAuth0(oidcUser.getClaims().get("sub").toString());
	}

	public User createIfNotExists(String sub, String email, String fullname, String profilepic) {
		if (getUser(sub).isEmpty()) {
			User user;
			if (email.equals(initalAdminEmail)) user = new Admin();
			else user = new Player();
			user.setAuth0(sub);
			user.setEmail(email);
			user.setName(fullname);
			user.setPicUrl(profilepic);
			return this.repository.save(user);
		}
		return null;
	}

	public Page<UserDTO> findAll(Pageable paging) {
		Page<User> pageUsers = this.repository.findAll(paging);
		int totalElements = (int) pageUsers.getTotalElements();
		return new PageImpl<>(pageUsers
				.stream()
				.map(UserDTO::new)
				.collect(Collectors.toList()), paging, totalElements);
	}

	public User getUser(int id) {
		return repository.findById(id).orElseThrow(UserNotFoundException::new);
	}

	public User setBlock(int id, boolean blocked) {
		User user = getUser(id);
		user.setBlocked(blocked);
		return repository.save(user);
	}

	public User updateUser(int id, String name, String email, String picPath, boolean paid, boolean blocked, String userType) {
		User user = getUser(id);
		user.setName(name);
		user.setEmail(email);
		user.setPicUrl(picPath);
		user.setPaid(paid);
		user.setBlocked(blocked);
		repository.save(user);
		repository.setDType(id, userType);
		return getUser(id);//return void maybe?
	}

	public void deleteUser(int id) {
		getUser(id); //throws exception when user doesn't exist.
		repository.deleteById(id);
	}

	public Page<UserDTO> findUserByName(String name, Pageable paging) {
		Page<User> pageUsers = this.repository.findUsersByNameContainingIgnoreCase(name, paging);
		int totalElements = (int) pageUsers.getTotalElements();
		return new PageImpl<>(pageUsers
				.stream()
				.map(UserDTO::new)
				.collect(Collectors.toList()), paging, totalElements);
	}

	public User updateUserProfile(UserRequestDTO userDTO, Integer id) {
		User user = repository.findById(id).orElseThrow(UserNotFoundException::new);
		user.setEmail(userDTO.getEmail());
		user.setName(userDTO.getName());
		return repository.save(user);

	}

	public List<PlayerDTO> getTopPoints() {
		List<PlayerDTO> playerList = new ArrayList<>();
		for (Player player : repository.getPlayers())
			playerList.add(new PlayerDTO(player.getName(), player.getScore(), player.getPicUrl()));

		playerList = playerList.stream().sorted(Comparator.comparing(PlayerDTO::getScore).reversed()).collect(Collectors.toList());

		if (playerList.size() < 10) return playerList;
		else return playerList.subList(0, 10);
	}
}
