package nl.v2b.lalaland.presentation;

import nl.v2b.lalaland.application.UserService;
import nl.v2b.lalaland.domain.User;
import nl.v2b.lalaland.presentation.dtos.BlockUserDto;
import nl.v2b.lalaland.presentation.dtos.PlayerDTO;
import nl.v2b.lalaland.presentation.dtos.UserDTO;
import nl.v2b.lalaland.presentation.dtos.UserRequestDTO;
import nl.v2b.lalaland.security.CurrentUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import jakarta.annotation.security.RolesAllowed;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/user")
public class UserController {
	private UserService userService;

	@Autowired
	public UserController(UserService userService) {
		this.userService = userService;
	}

	private Map<String, Object> genUserPageResponse(Page<UserDTO> page) {
		Map<String, Object> response = new HashMap<>();
		response.put("users", page.getContent());
		response.put("currentPage", page.getNumber());
		response.put("totalItems", page.getTotalElements());
		response.put("totalPages", page.getTotalPages());
		return response;
	}

	// In the URL parameter, follow the orderering of paramaters below
	// Example: api/user/all?page=1&?size=15
	@RolesAllowed("ADMIN")
	@GetMapping("/all")
	public ResponseEntity<Map<String, Object>> getAllUsers(
			@RequestParam(defaultValue = "0") int page,
			@RequestParam(defaultValue = "10") int size,
			@RequestParam(required = false) String name
	) {
		try {
			Pageable paging = PageRequest.of(page, size);

			Page<UserDTO> pageUsers;
			if (name == null)
				pageUsers = userService.findAll(paging);
			else
				pageUsers = userService.findUserByName(name, paging);

			Map<String, Object> response = genUserPageResponse(pageUsers);

			return new ResponseEntity<>(response, HttpStatus.OK);
		} catch (Exception e) {
			return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@GetMapping("/this")
	public ResponseEntity<UserDTO> getCurrentUser(@CurrentUser User user) {
		return new ResponseEntity<>(new UserDTO(userService.getUser(user.getId())), HttpStatus.OK);
	}

	@GetMapping("/{id}")
	@RolesAllowed("ADMIN")
	public UserDTO getUser(@PathVariable int id) {
			return new UserDTO(userService.getUser(id));
	}

	@PatchMapping("/{id}/block")
	@RolesAllowed("JUDGE")
	public UserDTO blockUser(@PathVariable int id, @RequestBody BlockUserDto blocked) {
		return new UserDTO(userService.setBlock(id, blocked.isBlocked()));
	}

	@PutMapping("/{id}")
	@RolesAllowed("ADMIN")
	public void updateUser(@PathVariable int id, @RequestBody UserRequestDTO dto) {
		userService.updateUser(id, dto.getName(), dto.getEmail(), dto.getPicture(), dto.isPaid(), dto.isBlocked(), dto.getUserType());
	}

    @DeleteMapping("/{id}")
    public ResponseEntity<Integer> deleteUser(@PathVariable int id) {
		userService.deleteUser(id);
		return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @PatchMapping("/{id}")
	@RolesAllowed("ADMIN")
    public void updateProfile(@PathVariable int id, @RequestBody UserRequestDTO dto){
		userService.updateUserProfile(dto, id);
	}

	@PatchMapping("/this")
	public void updateSelf(@CurrentUser User user, @RequestBody UserRequestDTO dto){
		userService.updateUserProfile(dto, user.getId());
	}

	@GetMapping("/topPoints")
	public ResponseEntity<List<PlayerDTO>> topPoints(){
		return new ResponseEntity<>(userService.getTopPoints(), HttpStatus.OK);
	}
}

