package nl.v2b.lalaland.security;

import lombok.extern.slf4j.Slf4j;
import nl.v2b.lalaland.application.UserService;
import nl.v2b.lalaland.domain.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityCustomizer;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.client.oidc.userinfo.OidcUserRequest;
import org.springframework.security.oauth2.client.oidc.userinfo.OidcUserService;
import org.springframework.security.oauth2.client.userinfo.OAuth2UserService;
import org.springframework.security.oauth2.core.oidc.user.DefaultOidcUser;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.logout.LogoutHandler;
import org.springframework.security.web.servlet.util.matcher.MvcRequestMatcher;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.web.servlet.handler.HandlerMappingIntrospector;

import java.util.*;

@Configuration
@Slf4j
@EnableWebSecurity
public class SecurityConfig {
    @Autowired
    private UserService service;
    private final LogoutHandler logoutHandler;

    public SecurityConfig(LogoutHandler logoutHandler) {
        this.logoutHandler = logoutHandler;
    }

    @Bean
    public SecurityFilterChain filterChain(HttpSecurity http) throws Exception {
        return http.csrf().disable().cors().and()
                .headers().frameOptions().disable().and()
                .authorizeHttpRequests((authz)->authz
                        .requestMatchers(new AntPathRequestMatcher("/api/**")).authenticated()
                        .requestMatchers(new AntPathRequestMatcher("/h2-console/**")).permitAll()
                        .requestMatchers(new MvcRequestMatcher.Builder(new HandlerMappingIntrospector()).pattern("/api/authorization/*")).permitAll()
                        .requestMatchers(new AntPathRequestMatcher("/api/logout")).permitAll()
                        .anyRequest().permitAll())
                .oauth2Login()
                .authorizationEndpoint().baseUri("/api/authorization").and()
                .redirectionEndpoint()
                .baseUri("/api/oauth2/callback/*")
                .and()
                .failureHandler(failureHandler())
                .successHandler(createUserOnLogin())
                .userInfoEndpoint()
                .oidcUserService(this.oidcUserService())
                .and().and().logout().deleteCookies("SESSION")
                // handle logout requests at /logout path
                .logoutRequestMatcher(new AntPathRequestMatcher("/api/logout"))
                // customize logout handler to log out of Auth0
                .addLogoutHandler(logoutHandler).and()
                .sessionManagement().invalidSessionStrategy(new MyInvalidSessionStrategy())
                .and().build();
    }
    @Bean
    public WebSecurityCustomizer webSecurityCustomizer() {
        return (web)->web.ignoring()
                .requestMatchers("/resources/**")
                .requestMatchers("/h2-console/**");
    }
    private OAuth2UserService<OidcUserRequest, OidcUser> oidcUserService() {
        final OidcUserService delegate = new OidcUserService();
        return userRequest -> {
            OidcUser oidcUser = delegate.loadUser(userRequest);
            Optional<User> optional=service.getUserOptional(oidcUser);
            List<SimpleGrantedAuthority> mappedAuthorities;
            if(optional.isEmpty()) {
                log.debug("new user logged in, assuming player, granting PLAYER role.");
                 mappedAuthorities=List.of(new SimpleGrantedAuthority("ROLE_PLAYER"));
            }
            else {
                User user=optional.get();
                mappedAuthorities=user.getAuthorities();
            }
            oidcUser = new DefaultOidcUser(mappedAuthorities, oidcUser.getIdToken(), oidcUser.getUserInfo());

            return oidcUser;
        };
    }


    @Bean
    public AuthenticationSuccessHandler createUserOnLogin(){
        return new CreateUserOnLogin();
    }
    @Bean
    public AuthenticationFailureHandler failureHandler(){
        return new FailureHandler();
    }
}
