package nl.v2b.lalaland.zoom;


import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.security.Keys;
import lombok.extern.slf4j.Slf4j;
import nl.v2b.lalaland.domain.*;
import nl.v2b.lalaland.storage.application.LalaFileService;
import nl.v2b.lalaland.storage.domain.LalaFile;
import nl.v2b.lalaland.zoom.dto.ZoomCreateDTO;
import nl.v2b.lalaland.zoom.dto.ZoomJoinDTO;
import nl.v2b.lalaland.zoom.dto.ZoomMeetingObjectDTO;
import nl.v2b.lalaland.zoom.dto.ZoomMeetingSettingsDTO;
import nl.v2b.lalaland.zoom.exceptions.CreateMeetingException;
import nl.v2b.lalaland.zoom.exceptions.HashNotSupportedException;
import nl.v2b.lalaland.zoom.exceptions.MeetingNotFoundException;
import nl.v2b.lalaland.zoom.exceptions.UnknownRoleException;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import jakarta.transaction.Transactional;
import jakarta.xml.bind.DatatypeConverter;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Base64;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Service("zoomService")
@Transactional
@Slf4j
public class ZoomMeetingService {
    @Value("${zoom.apiKey}")
    private String apiKey;
    @Value("${zoom.apiSecret}")
    private String apiSecret;
    @Value("${zoom.userId}")
    private String userId;
    private final LalaFileService lalaFileService;
    private final SpringZoomMeetingRepository repository;

    public ZoomMeetingService(SpringZoomMeetingRepository repository, LalaFileService lalaFileService){
        this.repository = repository;
        this.lalaFileService=lalaFileService;
    }
    public Zoom saveMeeting(Zoom zoom){
        return repository.save(zoom);
    }
    public Zoom getMeeting(int id){
        return repository.findById(id).orElseThrow(MeetingNotFoundException::new);
    }
    public String generateSignature(Zoom zoom, int role) {
        Mac hasher = null;
        try {
            hasher = Mac.getInstance("HmacSHA256");
        } catch (NoSuchAlgorithmException e) {
            log.error("Zoom hashing algorithm not supported(HmacSHA256).");
            throw new HashNotSupportedException();
        }
        String ts = Long.toString(System.currentTimeMillis() - 30000);
        String msg = String.format("%s%s%s%d", apiKey, zoom.getMeetingNumber(), ts, role);

        try {
            hasher.init(new SecretKeySpec(apiSecret.getBytes(), "HmacSHA256"));
        } catch (InvalidKeyException e) {
            log.error("Unexpected error, unable to create signature for zoom.");
            throw new nl.v2b.lalaland.zoom.exceptions.InvalidKeyException();
        }

        String message = Base64.getEncoder().encodeToString(msg.getBytes());
        byte[] hash = hasher.doFinal(message.getBytes());

        String hashBase64Str = DatatypeConverter.printBase64Binary(hash);
        String tmpString = String.format("%s.%s.%s.%d.%s", apiKey, zoom.getMeetingNumber(), ts, role, hashBase64Str);
        String encodedString = Base64.getEncoder().encodeToString(tmpString.getBytes());

        return encodedString.replaceAll("\\=+$", "");
    }
    public ZoomJoinDTO generateZoomJoinDTO(User user, int zoomId) {
        Zoom zoom=getMeeting(zoomId);
        int role=-1;
        if(user instanceof Player) role=0;
        if(user instanceof Admin) role=1;
        if(user instanceof Judge) role=1;
        if(role==-1) throw new UnknownRoleException();
        return new ZoomJoinDTO(zoom,generateSignature(zoom,role),apiKey, user);
    }
    public String generateZoomJWTTOken() {
        String id = UUID.randomUUID().toString().replace("-", "");
        SignatureAlgorithm signatureAlgorithm = SignatureAlgorithm.HS256;

        Date creation = new Date(System.currentTimeMillis());
        Date tokenExpiry = new Date(System.currentTimeMillis() + (1000 *60*60*24*365)); //1 year

        Key key = Keys
                .hmacShaKeyFor(apiSecret.getBytes());
        return Jwts.builder()
                .setId(id)
                .setIssuer(apiKey)
                .setIssuedAt(creation)
                .setSubject("")
                .setExpiration(tokenExpiry)
                .signWith(key, signatureAlgorithm)
                .compact();
    }
    public Zoom createMeeting(ZoomCreateDTO dto) {
        String topic=dto.getTopic();
        LalaFile file=null;
        if(dto.getFile()!=null) file= lalaFileService.get(dto.getFile());
        ZoomMeetingObjectDTO zoomMeetingObjectDTO = new ZoomMeetingObjectDTO();
        String apiUrl = "https://api.zoom.us/v2/users/" + userId + "/meetings";
        zoomMeetingObjectDTO.setPassword(RandomStringUtils.random(10,0,0,true,true,null,new SecureRandom()));
        ZoomMeetingSettingsDTO settingsDTO = new ZoomMeetingSettingsDTO();
        settingsDTO.setJoin_before_host(true);
        settingsDTO.setParticipant_video(true);
        settingsDTO.setHost_video(false);
        settingsDTO.setAuto_recording("none");
        settingsDTO.setMute_upon_entry(false);
        zoomMeetingObjectDTO.setSettings(settingsDTO);
        zoomMeetingObjectDTO.setTopic(topic);
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", "Bearer " + generateZoomJWTTOken());
        headers.add("content-type", "application/json");
        HttpEntity<ZoomMeetingObjectDTO> httpEntity = new HttpEntity<>(zoomMeetingObjectDTO, headers);
        ResponseEntity<ZoomMeetingObjectDTO> zEntity = restTemplate.exchange(apiUrl, HttpMethod.POST, httpEntity, ZoomMeetingObjectDTO.class);
        ZoomMeetingObjectDTO body=zEntity.getBody();
        if(body==null) throw new CreateMeetingException();
        Zoom zoom = new Zoom(body.getId(),body.getPassword(),topic,file);
        return saveMeeting(zoom);
    }
    public List<Zoom> getAll() {
        return repository.findAll();
    }
    public void delete(int id) {
        repository.delete(getMeeting(id));
    }
}
