package nl.v2b.lalaland.presentation;

import com.jayway.jsonpath.JsonPath;
import nl.v2b.lalaland.application.UserService;
import nl.v2b.lalaland.security.SecurityHelper;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.RequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;

import static org.hamcrest.Matchers.is;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;

@SpringBootTest
@AutoConfigureMockMvc
class UserControllerIntegrationTest {
	@Autowired
	private MockMvc mockMvc;

	@Autowired
	private UserService userService;

	@Autowired
	private SecurityHelper helper;
	private int id;
	private int fakeId;
	private RequestBuilder getUser;
	private RequestBuilder fakeUser;

	@BeforeEach
	void construct() {
		this.fakeId = 1000000000;
		this.id = userService.createIfNotExists("exampleSub", "example@example.com",
				"example person", "example.jpg").getId();

		this.getUser = MockMvcRequestBuilders
				.get("/api/user/" + this.id).with(helper.oidcAdmin());

		this.fakeUser = MockMvcRequestBuilders
				.get("/api/user/" + this.fakeId).with(helper.oidcAdmin());
	}

	@AfterEach()
    void decontsruct(){
	    this.userService.deleteUser(this.id);
    }

	@Test
	@DisplayName("Get all users returns 200 response code.")
	void getUsersTest() throws Exception {
		RequestBuilder request = MockMvcRequestBuilders
				.get("/api/user/all").with(helper.oidcAdmin());
		mockMvc.perform(request).andExpect(status().isOk());
	}

	@Test
	@DisplayName("Get specific user returns 200 response code.")
	void getUserSuccesfullyTest() throws Exception {
		mockMvc.perform(this.getUser)
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.fullname", is("example person")))
				.andExpect(jsonPath("$.email", is("example@example.com")))
				.andExpect(jsonPath("$.picture", is("example.jpg")))
				.andExpect(jsonPath("$.paid", is(false)))
				.andExpect(jsonPath("$.blocked", is(false)));
	}

	@Test
	@DisplayName("Get specific user returns 404 response code.")
	void getUserFailTest() throws Exception {
		mockMvc.perform(this.fakeUser)
				.andExpect((status().isNotFound()));
	}

	@Test
	@DisplayName("User gets deleted with right responsecode")
	void deleteUserSuccessfullyTest() throws Exception {
        int user2id = userService.createIfNotExists("exampleSub2", "example2@example.com",
                "example person2", "example2.jpg").getId();

		RequestBuilder deleteRequest = MockMvcRequestBuilders
				.delete("/api/user/" + user2id).with(helper.oidcAdmin());
		mockMvc.perform(deleteRequest).andExpect(status().isNoContent());
	}

	@Test
	@DisplayName("User doesnt get deleted with right responsecode")
	void deleteUserFailTest() throws Exception {
		RequestBuilder request = MockMvcRequestBuilders
				.delete("/api/user/" + this.fakeId).with(helper.oidcAdmin());
		mockMvc.perform(request).andExpect(status().isNotFound());
	}

	@Test
	@DisplayName("User gets blocked")
	void blockUserSuccessfullyTest() throws Exception {
		RequestBuilder request = MockMvcRequestBuilders
				.patch("/api/user/" + this.id + "/block").contentType(MediaType.APPLICATION_JSON).content("{\"blocked\":\"true\"}").with(helper.oidcAdmin());
		MvcResult result = mockMvc.perform(request).andExpect(status().isOk()).andReturn();
		assertEquals(true, JsonPath.read(result.getResponse().getContentAsString(), "$.blocked"));

	}

	@Test
	@DisplayName("User gets blocked")
	void blockUserFailTest() throws Exception {
		RequestBuilder request = MockMvcRequestBuilders
				.patch("/api/user/" + this.fakeId + "/block").contentType(MediaType.APPLICATION_JSON).content("{\"blocked\":\"true\"}").with(helper.oidcAdmin());
		mockMvc.perform(request).andExpect(status().isNotFound());
	}

	@Test
	@DisplayName("User gets updated with right responsecode")
	void updateUserSuccessfullyTest() throws Exception {
		RequestBuilder updateRequest1 = MockMvcRequestBuilders
				.put("/api/user/" + this.id).contentType(MediaType.APPLICATION_JSON).content("{\"name\":\"Test\",\"email\":\"name@email.com\",\"picture\":\"http://blabla.nl/image.png\",\"paid\":\"true\",\"userType\":\"Admin\",\"blocked\":\"false\"}").with(helper.oidcAdmin());

		mockMvc.perform(updateRequest1)
				.andExpect(status().isOk());

		mockMvc.perform(this.getUser)
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.fullname", is("Test")))
				.andExpect(jsonPath("$.email", is("name@email.com")))
				.andExpect(jsonPath("$.picture", is("http://blabla.nl/image.png")))
				.andExpect(jsonPath("$.paid", is(true)))
				.andExpect(jsonPath("$.blocked", is(false)));

	}

	@Test
	@DisplayName("User gets updated with right responsecode")
	void updateUserFailTest() throws Exception {
		RequestBuilder fakeUpdateRequest = MockMvcRequestBuilders
				.put("/api/user/" + this.fakeId).contentType(MediaType.APPLICATION_JSON).content("{\"name\":\"Test\",\"email\":\"name@email.com\",\"picture\":\"http://blabla.nl/image.png\",\"paid\":\"true\",\"userType\":\"Admin\",\"blocked\":\"false\"}").with(helper.oidcAdmin());

		mockMvc.perform(fakeUpdateRequest)
				.andExpect(status().isNotFound());
	}
}
